import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()
_CONTEXT.TYPE_EXCEPTIONS.append("doc")


class doc(jsg.JSGObject):
    _reference_types = []
    _members = {'street': jsg.String,
                'no': jsg.Integer}
    _strict = True

    def __init__(self,
                 street: str = None,
                 no: int = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.street = street
        self.no = no


_CONTEXT.NAMESPACE = locals()




import unittest

from jsonasobj import loads as jsonloads
from jsonasobj.jsonobj import as_json

from pyjsg.jsglib import JSGObjectMap, JSGContext, ArrayFactory, Integer
from tests.test_jsglib.iri_defn import *

_CONTEXT = JSGContext()


class ObjectMapTestCase(unittest.TestCase):

    def test_basic_map(self):
        class IntObjectMap(JSGObjectMap):
            _name_filter = HEX
            _value_type = ArrayFactory('', _CONTEXT, Integer, 0, None)

            def __init__(self,
                         **_kwargs):
                super().__init__(_CONTEXT, **_kwargs)


        x = IntObjectMap()
        x.E = [1,2,3]
        self.assertTrue(x._is_valid())
        self.assertEqual(as_json(x), as_json(jsonloads('{"E":[1,2,3]}')))
        with self.assertRaises(ValueError):
            x.G = [1, 2, 4]
        with self.assertRaises(ValueError):
            x.C = 1
        with self.assertRaises(ValueError):
            x = IntObjectMap(aa=[1])

    def test_any_key(self):
        class ALPHA(JSGString):
            pattern = JSGPattern(r'[A-Za-z]')

        class AnyKeyObjectMap(JSGObjectMap):
            _value_type = ALPHA

            def __init__(self,
                         **_kwargs):
                super().__init__(_CONTEXT, **_kwargs)

        x = AnyKeyObjectMap(**dict(I1="a"))
        x["item 2"] = "b"
        self.assertTrue(x._is_valid())
        with self.assertRaises(ValueError):
            x.c = "1"

    def test_any_value(self):
        class IRIKey(JSGObjectMap):
            _name_filter = IRI

            def __init__(self,
                         **_kwargs):
                super().__init__(_CONTEXT, **_kwargs)

        x = IRIKey(**{"http://example.org": 42, "http://ex.org?id=1": None})
        self.assertEqual('{"http://example.org": 42, "http://ex.org?id=1": null}', as_json(x, indent=None))
        self.assertTrue(x._is_valid())
        self.assertEqual(42, x["http://example.org"])
        self.assertIsNone(x["http://ex.org?id=1"])


if __name__ == '__main__':
    unittest.main()




import unittest

from pyjsg.jsglib import JSGString, JSGPattern


class Issue9TestCase(unittest.TestCase):
    def test_issue_9(self):
        self.assertTrue(isinstance("abc", JSGString))
        self.assertTrue(isinstance("", JSGString))
        self.assertFalse(isinstance(None, JSGString))
        self.assertFalse(isinstance(1, JSGString))
        self.assertFalse(isinstance([], JSGString))
        self.assertTrue(JSGString("abc"), JSGString)

        class T(JSGString):
            pass
        self.assertTrue(T("abc"), JSGString)

        class TP(JSGString):
            pattern = JSGPattern("[a-zA-Z]+")
        self.assertTrue(TP("abc"), JSGString)


if __name__ == '__main__':
    unittest.main()



import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()
_CONTEXT.TYPE_EXCEPTIONS.append("Wildcard")
_CONTEXT.TYPE_EXCEPTIONS.append("ObjectLiteral")
_CONTEXT.TYPE_EXCEPTIONS.append("Stem")
_CONTEXT.TYPE_EXCEPTIONS.append("StemRange")


class IRI(jsg.String):
    pattern = jsg.JSGPattern(r'')


class Wildcard(jsg.JSGObject):
    _reference_types = []
    _members = {}
    _strict = True

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)



class ObjectLiteral(jsg.JSGObject):
    _reference_types = []
    _members = {'value': jsg.String,
                'language': typing.Optional[jsg.String],
                'type': typing.Optional[jsg.String]}
    _strict = True

    def __init__(self,
                 value: str = None,
                 language: typing.Optional[str] = None,
                 type: typing.Optional[str] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.value = value
        self.language = language
        self.type = type



objectValue = typing.Union[IRI, ObjectLiteral]


class Stem(jsg.JSGObject):
    _reference_types = []
    _members = {'stem': IRI}
    _strict = True

    def __init__(self,
                 stem: str = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.stem = stem



class StemRange(jsg.JSGObject):
    _reference_types = []
    _members = {'stem': typing.Union[IRI, Wildcard],
                'exclusions': typing.Optional[jsg.ArrayFactory('exclusions', _CONTEXT, typing.Union[typing.Union[IRI, ObjectLiteral], Stem], 1, None)]}
    _strict = True

    def __init__(self,
                 stem: typing.Union[str, Wildcard] = None,
                 exclusions: typing.Optional[typing.List[typing.Union[typing.Union[str, ObjectLiteral], Stem]]] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.stem = stem
        self.exclusions = exclusions


_CONTEXT.NAMESPACE = locals()



import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()
_CONTEXT.TYPE = "id"


class empty_object(jsg.JSGObject):
    _reference_types = []
    _members = {}
    _strict = True

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)



class wild_card_object(jsg.JSGObject):
    _reference_types = []
    _members = {}
    _strict = False

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)



class closed_object(jsg.JSGObject):
    _reference_types = []
    _members = {'a': jsg.String,
                'b': typing.Optional[jsg.Integer]}
    _strict = True

    def __init__(self,
                 a: str = None,
                 b: typing.Optional[int] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.a = a
        self.b = b



class open_object(jsg.JSGObject):
    _reference_types = []
    _members = {'a': jsg.String,
                'b': typing.Optional[jsg.Integer]}
    _strict = False

    def __init__(self,
                 a: str = None,
                 b: typing.Optional[int] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.a = a
        self.b = b



class object_options_1_(jsg.JSGObject):
    _reference_types = []
    _members = {'a': jsg.String,
                'b': typing.Optional[jsg.Integer]}
    _strict = True

    def __init__(self,
                 a: str = None,
                 b: typing.Optional[int] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.a = a
        self.b = b



class object_options_2_(jsg.JSGObject):
    _reference_types = []
    _members = {'k': jsg.JSGNull}
    _strict = False

    def __init__(self,
                 k: type(None) = jsg.Empty,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.k = k



class object_options_3_(jsg.JSGObject):
    _reference_types = []
    _members = {}
    _strict = True

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)



class object_options_2_1_(jsg.JSGObject):
    _reference_types = []
    _members = {'a': jsg.String}
    _strict = True

    def __init__(self,
                 a: str = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.a = a



class object_options_2_2_(jsg.JSGObject):
    _reference_types = []
    _members = {}
    _strict = True

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)



class object_options(jsg.JSGObject):
    _reference_types = [object_options_1_, object_options_2_, object_options_3_]
    _members = {'a': typing.Optional[jsg.String],
                'b': typing.Optional[jsg.Integer],
                'k': typing.Optional[jsg.JSGNull]}
    _strict = True

    def __init__(self,
                 opts_: typing.Union[object_options_1_, object_options_2_, object_options_3_] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        if opts_ is not None:
            if isinstance(opts_, object_options_1_):
                self.a = opts_.a
                self.b = opts_.b
            elif isinstance(opts_, object_options_2_):
                self.k = opts_.k
            elif isinstance(opts_, object_options_3_):
                pass
            else:
                raise ValueError(f"Unrecognized value type: {opts_}")



class object_options_2(jsg.JSGObject):
    _reference_types = [object_options_2_1_, object_options_2_2_]
    _members = {'a': typing.Optional[jsg.String]}
    _strict = True

    def __init__(self,
                 opts_: typing.Union[object_options_2_1_, object_options_2_2_] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        if opts_ is not None:
            if isinstance(opts_, object_options_2_1_):
                self.a = opts_.a
            elif isinstance(opts_, object_options_2_2_):
                pass
            else:
                raise ValueError(f"Unrecognized value type: {opts_}")



class doc(jsg.JSGObject):
    _reference_types = []
    _members = {'e': typing.Union[empty_object, wild_card_object, closed_object, open_object, object_options, object_options_2]}
    _strict = True

    def __init__(self,
                 e: typing.Union[empty_object, wild_card_object, closed_object, open_object, object_options, object_options_2] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.e = e


_CONTEXT.NAMESPACE = locals()



import unittest
from dataclasses import dataclass
from typing import cast, List, Tuple, Optional

from pyjsg.parser_impl.jsg_doc_parser import JSGDocParser
from pyjsg.parser_impl.jsg_objectexpr_parser import JSGObjectExpr
from tests.test_basics.parser import parse


@dataclass
class TestEntry:
    text: str
    name: str
    deps: List[str]
    sigs: List[str]
    membs: List[Tuple[str, str]]
    inits: List[str]

    @staticmethod
    def gen_entry(t: JSGObjectExpr, text: Optional[str]=None) -> str:
        return str(TestEntry(text if text else t.text, str(t),
                             t.dependency_list(), t.signatures(), t.members_entries(), t.initializers()))


i0 = TestEntry(text='a {}', name='objectExpr: simple object', deps=[], sigs=[], membs=[], inits=[])
i1 = TestEntry(text='a {,}', name='objectExpr: simple object', deps=[], sigs=[], membs=[], inits=[])
i2 = TestEntry(text='a {a: @int}', name='objectExpr: simple object', deps=[], sigs=['a: int = None'], 
               membs=[('a', 'jsg.Integer')], inits=['self.a = a'])
i3 = TestEntry(text='a {b: .}', name='objectExpr: simple object', deps=[], sigs=['b: object = jsg.Empty'],
               membs=[('b', "jsg.AnyTypeFactory('b', _CONTEXT)")], inits=['self.b = b'])
i4 = TestEntry(text='a {b: .? c: . d: .* e: .+ f: a}', 
               name='objectExpr: simple object', 
               deps=['a'], 
               sigs=['b: typing.Optional[object] = jsg.Empty', 'c: object = jsg.Empty',
                     'd: typing.List[object] = None',
                     'e: typing.List[object] = None', 'f: a = None'],
               membs=[('b', "typing.Optional[jsg.AnyTypeFactory('b', _CONTEXT)]"),
                      ('c', "jsg.AnyTypeFactory('c', _CONTEXT)"),
                      ('d', "jsg.ArrayFactory('d', _CONTEXT, jsg.AnyTypeFactory('d', _CONTEXT), 0, None)"),
                      ('e', "jsg.ArrayFactory('e', _CONTEXT, jsg.AnyTypeFactory('e', _CONTEXT), 1, None)"),
                      ('f', 'a')], 
               inits=['self.b = b',
                      'self.c = c',
                      "self.d = d",
                      "self.e = e",
                      'self.f = f'])
i5 = TestEntry(text='a {b: @int c:@string |, }', 
               name='objectExpr: object choices', 
               deps=['a_1_', 'a_2_'], 
               sigs=['opts_: typing.Union[a_1_, a_2_] = None'],
               membs=[('b', 'typing.Optional[jsg.Integer]'), ('c', 'typing.Optional[jsg.String]')],
               inits=['if opts_ is not None:', '    if isinstance(opts_, a_1_):', '        self.b = opts_.b',
                      '        self.c = opts_.c', '    elif isinstance(opts_, a_2_):', '        pass', '    else:',
                      '        raise ValueError(f"Unrecognized value type: {opts_}")'])
i6 = TestEntry(text='a {b: . | c: .}', 
               name='objectExpr: object choices', 
               deps=['a_1_', 'a_2_'], 
               sigs=['opts_: typing.Union[a_1_, a_2_] = None'],
               membs=[('b', "typing.Optional[jsg.AnyTypeFactory('b', _CONTEXT)]"),
                      ('c', "typing.Optional[jsg.AnyTypeFactory('c', _CONTEXT)]")],
               inits=['if opts_ is not None:', '    if isinstance(opts_, a_1_):', '        self.b = opts_.b',
                      '    elif isinstance(opts_, a_2_):', '        self.c = opts_.c', '    else:',
                      '        raise ValueError(f"Unrecognized value type: {opts_}")'])
i7 = TestEntry(text='a {b: .? c: . | d: .*, e: .+ | f: a, }', 
               name='objectExpr: object choices', 
               deps=['a_1_', 'a_2_', 'a', 'a_3_'], 
               sigs=['opts_: typing.Union[a_1_, a_2_, a_3_] = None'],
               membs=[('b', "typing.Optional[jsg.AnyTypeFactory('b', _CONTEXT)]"),
                      ('c', "typing.Optional[jsg.AnyTypeFactory('c', _CONTEXT)]"),
                      ('d',
                       "typing.Optional[jsg.ArrayFactory('d', _CONTEXT, jsg.AnyTypeFactory('d', _CONTEXT), 0, None)]"),
                      ('e',
                       "typing.Optional[jsg.ArrayFactory('e', _CONTEXT, jsg.AnyTypeFactory('e', _CONTEXT), 1, None)]"),
                      ('f', 'typing.Optional[a]')],
               inits=['if opts_ is not None:', '    if isinstance(opts_, a_1_):', '        self.b = opts_.b',
                      '        self.c = opts_.c', '    elif isinstance(opts_, a_2_):', '        self.d = opts_.d',
                      '        self.e = opts_.e', '    elif isinstance(opts_, a_3_):', '        self.f = opts_.f',
                      '    else:',
                      '        raise ValueError(f"Unrecognized value type: {opts_}")'])


o0 = TestEntry(text='{}', name='objectExpr: simple object', deps=[], sigs=[], membs=[], inits=[])
o1 = TestEntry(text='{,}', name='objectExpr: simple object', deps=[], sigs=[], membs=[], inits=[])
o2 = TestEntry(text='{a: @int}', name='objectExpr: simple object', deps=[], sigs=['a: int = None'], 
               membs=[('a', 'jsg.Integer')], inits=['self.a = a'])
o3 = TestEntry(text='{b: .}', name='objectExpr: simple object', deps=[], sigs=['b: object = jsg.Empty'],
               membs=[('b', "jsg.AnyTypeFactory('b', _CONTEXT)")], inits=['self.b = b'])
o4 = TestEntry(text='{b: .? c: . d: .* e: .+ f: a}', 
               name='objectExpr: simple object', 
               deps=['a'], 
               sigs=['b: typing.Optional[object] = jsg.Empty', 'c: object = jsg.Empty',
                     'd: typing.List[object] = None',
                     'e: typing.List[object] = None', 'f: Undefined(a) = None'],
               membs=[('b', "typing.Optional[jsg.AnyTypeFactory('b', _CONTEXT)]"),
                      ('c', "jsg.AnyTypeFactory('c', _CONTEXT)"),
                      ('d', "jsg.ArrayFactory('d', _CONTEXT, jsg.AnyTypeFactory('d', _CONTEXT), 0, None)"),
                      ('e', "jsg.ArrayFactory('e', _CONTEXT, jsg.AnyTypeFactory('e', _CONTEXT), 1, None)"),
                      ('f', 'Undefined(a)')],
               inits=['self.b = b',
                      'self.c = c',
                      "self.d = d",
                      "self.e = e",
                      'self.f = f'])
o5 = TestEntry(text='{b: @int c:@string |, }', 
               name='objectExpr: object choices', 
               deps=['_Anon1_1_', '_Anon1_2_'],
               sigs=['opts_: typing.Union[_Anon1_1_, _Anon1_2_] = None'],
               membs=[('b', 'typing.Optional[jsg.Integer]'), ('c', 'typing.Optional[jsg.String]')],
               inits=['if opts_ is not None:', '    if isinstance(opts_, _Anon1_1_):', '        self.b = opts_.b',
                      '        self.c = opts_.c', '    elif isinstance(opts_, _Anon1_2_):', '        pass', '    else:',
                      '        raise ValueError(f"Unrecognized value type: {opts_}")'])
o6 = TestEntry(text='{b: . | c: .}', 
               name='objectExpr: object choices', 
               deps=['_Anon1_1_', '_Anon1_2_'], 
               sigs=['opts_: typing.Union[_Anon1_1_, _Anon1_2_] = None'],
               membs=[('b', "typing.Optional[jsg.AnyTypeFactory('b', _CONTEXT)]"),
                      ('c', "typing.Optional[jsg.AnyTypeFactory('c', _CONTEXT)]")],
               inits=['if opts_ is not None:', '    if isinstance(opts_, _Anon1_1_):', '        self.b = opts_.b',
                      '    elif isinstance(opts_, _Anon1_2_):', '        self.c = opts_.c', '    else:',
                      '        raise ValueError(f"Unrecognized value type: {opts_}")'])
o7 = TestEntry(text='{b: .? c: . | d: .*, e: .+ | f: a, }', 
               name='objectExpr: object choices', 
               deps=['_Anon1_1_', '_Anon1_2_', 'a', '_Anon1_3_'], 
               sigs=['opts_: typing.Union[_Anon1_1_, _Anon1_2_, _Anon1_3_] = None'],
               membs=[('b', "typing.Optional[jsg.AnyTypeFactory('b', _CONTEXT)]"),
                      ('c', "typing.Optional[jsg.AnyTypeFactory('c', _CONTEXT)]"),
                      ('d',
                       "typing.Optional[jsg.ArrayFactory('d', _CONTEXT, jsg.AnyTypeFactory('d', _CONTEXT), 0, None)]"),
                      ('e',
                       "typing.Optional[jsg.ArrayFactory('e', _CONTEXT, jsg.AnyTypeFactory('e', _CONTEXT), 1, None)]"),
                      ('f', 'typing.Optional[Undefined(a)]')],
               inits=['if opts_ is not None:', '    if isinstance(opts_, _Anon1_1_):', '        self.b = opts_.b',
                      '        self.c = opts_.c', '    elif isinstance(opts_, _Anon1_2_):', '        self.d = opts_.d',
                      '        self.e = opts_.e', '    elif isinstance(opts_, _Anon1_3_):', '        self.f = opts_.f',
                      '    else:',
                      '        raise ValueError(f"Unrecognized value type: {opts_}")'])

test_entries: List[Tuple[str, TestEntry, TestEntry]] = [
    ('{}', i0, o0),
    ('{,}', i1, o1),
    ('{a: @int}', i2, o2),
    ('{b: .}', i3, o3),
    ('{b: .? c: . d: .* e: .+ f: a}', i4, o4),
    ('{b: @int c:@string |, }', i5, o5),
    ('{b: . | c: .}', i6, o6),
    ('{b: .? c: . | d: .*, e: .+ | f: a, }', i7, o7),
]


class ObjectExprParserTestCase(unittest.TestCase):
    def test_basics(self):
        d = cast(JSGDocParser, parse('a ' + test_entries[0][0], "objectDef", JSGDocParser))
        t = d._context.reference('a')
        self.assertEqual("None", t.mt_value())

        for te in test_entries:
            text = "a " + te[0]
            e = te[1]
            d = cast(JSGDocParser, parse(text, "objectDef", JSGDocParser))
            self.assertIsNotNone(d, f"Parse error: {text}")
            t = d._context.reference('a')
            self.assertEqual(e.name, str(t))
            self.assertEqual('a', t.signature_type(), text)
            self.assertEqual('a', t.python_type(), text)
            self.assertEqual(e.deps, t.dependency_list(), text)
            self.assertEqual(e.sigs, t.signatures(), text)
            self.assertEqual(e.membs, t.members_entries(), text)
            self.assertEqual(e.inits, t.initializers(), text)

    def test_anonymous_entries(self):
        for te in test_entries:
            e = te[2]
            t = cast(JSGObjectExpr, parse(te[0], "objectExpr", JSGObjectExpr))
            self.assertIsNotNone(t, f"Parse error: {e.text}")
            self.assertEqual(e.name, str(t))
            self.assertEqual('_Anon1', t.signature_type(), e.text)
            self.assertEqual('_Anon1', t.python_type(), e.text)
            self.assertEqual(e.deps, t.dependency_list(), e.text)
            self.assertEqual(e.sigs, t.signatures(), e.text)
            self.assertEqual(e.membs, t.members_entries(), e.text)
            self.assertEqual(e.inits, t.initializers(), e.text)

    def test_opt_choice_branch(self):
        text = '{id: @string |}'
        t = cast(JSGObjectExpr, parse(text, 'objectExpr', JSGObjectExpr))
        self.assertIsNotNone(t, f"Parse error")
        self.assertEqual('objectExpr: object choices', str(t))
        self.assertEqual('_Anon1', t.signature_type(), text)
        self.assertEqual('_Anon1', t.python_type(), text)
        self.assertEqual(['_Anon1_1_', '_Anon1_2_'], t.dependency_list(), text)
        self.assertEqual(['opts_: typing.Union[_Anon1_1_, _Anon1_2_] = None'], t.signatures(), text)
        self.assertEqual([('id', 'typing.Optional[jsg.String]')], t.members_entries(), text)
        self.assertEqual([
            'if opts_ is not None:',
            '    if isinstance(opts_, _Anon1_1_):',
            '        self.id = opts_.id',
            '    elif isinstance(opts_, _Anon1_2_):',
            '        pass',
            '    else:',
            '        raise ValueError(f"Unrecognized value type: {opts_}")'], t.initializers(), text)


if __name__ == '__main__':
    unittest.main()



# Auto generated from JSGPython by PyJSG version 0.8b1
# Generation date: 2018-07-31 11:27
#
import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()
_CONTEXT.TYPE_EXCEPTIONS.append("doc")


class doc(jsg.JSGObject):
    _reference_types = []
    _members = {'opt': typing.Optional[jsg.String],
                'req': jsg.String,
                'l0n': jsg.ArrayFactory('l0n', _CONTEXT, jsg.String, 0, None),
                'l1n': jsg.ArrayFactory('l1n', _CONTEXT, jsg.String, 1, None),
                'l01': jsg.ArrayFactory('l01', _CONTEXT, jsg.String, 0, 1),
                'l11': jsg.ArrayFactory('l11', _CONTEXT, jsg.String, 0, None),
                'l0na': jsg.ArrayFactory('l0na', _CONTEXT, jsg.String, 0, None),
                'l1na': jsg.ArrayFactory('l1na', _CONTEXT, jsg.String, 1, None),
                'optl0n': typing.Optional[jsg.ArrayFactory('optl0n', _CONTEXT, jsg.String, 0, None)],
                'optl1n': typing.Optional[jsg.ArrayFactory('optl1n', _CONTEXT, jsg.String, 1, None)]}
    _strict = True

    def __init__(self,
                 opt: typing.Optional[str] = None,
                 req: str = None,
                 l0n: typing.List[str] = None,
                 l1n: typing.List[str] = None,
                 l01: typing.List[str] = None,
                 l11: typing.List[str] = None,
                 l0na: typing.List[str] = None,
                 l1na: typing.List[str] = None,
                 optl0n: typing.Optional[typing.List[str]] = None,
                 optl1n: typing.Optional[typing.List[str]] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.opt = opt
        self.req = req
        self.l0n = l0n
        self.l1n = l1n
        self.l01 = l01
        self.l11 = l11
        self.l0na = l0na
        self.l1na = l1na
        self.optl0n = optl0n
        self.optl1n = optl1n


_CONTEXT.NAMESPACE = locals()



from typing import Callable, Optional

from antlr4 import InputStream, CommonTokenStream

from pyjsg.parser.jsgLexer import jsgLexer
from pyjsg.parser.jsgParser import jsgParser
from pyjsg.parser.jsgParserVisitor import jsgParserVisitor
from pyjsg.parser_impl.generate_python import ParseErrorListener
from pyjsg.parser_impl.jsg_doc_context import JSGDocContext


def parse(text: str, production_rule: str, listener) -> Optional[jsgParserVisitor]:
    """ 
    Parse text fragment according to supplied production rule and evaluate with listener class.
    
    Example: parse("{1,*}", "ebnfSuffix", JSGEbnf)
    """
    error_listener = ParseErrorListener()
    lexer = jsgLexer(InputStream(text))
    lexer.addErrorListener(error_listener)
    tokens = CommonTokenStream(lexer)
    tokens.fill()
    if error_listener.n_errors:
        return None
    parser = jsgParser(tokens)
    parser.addErrorListener(error_listener)
    base_node = getattr(parser, production_rule)()
    listener_module = listener(JSGDocContext())
    listener_module.visit(base_node)
    return listener_module if not error_listener.n_errors else None



# Auto generated from JSGPython by PyJSG version 0.8b1
# Generation date: 2018-07-31 11:32
#
import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()
_CONTEXT.TYPE_EXCEPTIONS.append("Person")
_CONTEXT.TYPE_EXCEPTIONS.append("Directory")


class PNAME(jsg.JSGString):
    pattern = jsg.JSGPattern(r'[A-Z][0-9]+')


class Person(jsg.JSGObject):
    _reference_types = []
    _members = {'name': jsg.String,
                'age': typing.Optional[jsg.Integer]}
    _strict = True

    def __init__(self,
                 name: str = None,
                 age: typing.Optional[int] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.name = name
        self.age = age



class Directory(jsg.JSGObjectMap):
    _name_filter = PNAME
    _value_type = Person

    def __init__(self,
                 **_kwargs):
        super().__init__(_CONTEXT, **_kwargs)

_CONTEXT.NAMESPACE = locals()



# Auto generated from JSGPython by PyJSG version 0.8b1
# Generation date: 2018-07-31 11:33
#
import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()
_CONTEXT.TYPE_EXCEPTIONS.append("l")


class l(jsg.JSGObject):
    _reference_types = []
    _members = {'e': jsg.ArrayFactory('e', _CONTEXT, jsg.String, 1, None)}
    _strict = True

    def __init__(self,
                 e: typing.List[str] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.e = e


_CONTEXT.NAMESPACE = locals()



import unittest
from io import StringIO
from types import ModuleType
from typing import cast, TextIO

from pyjsg.jsglib.loader import loads, Logger, is_valid
from pyjsg.parser_impl.generate_python import parse


class Harness(unittest.TestCase):

    def do_test(self, jsg: str, json: str, should_pass: bool=True, _: bool=True,
                fails_validation: bool=False, expected: str=None) \
            -> None:
        """ Validate JSON against JSG

        :param jsg: JSG definition
        :param json: JSON to validate
        :param should_pass: True means expect success
        :param jsg_exception: (unused)
        :param fails_validation: True means fails is_valid, false means raises exception
        :param expected:
        :return:
        """
        python = parse(jsg, self.__class__.__name__)
        spec = compile(python, self.__class__.__name__, 'exec')
        module = ModuleType(self.__class__.__name__)
        exec(spec, module.__dict__)

        if should_pass:
            self.assertTrue(is_valid(loads(json, module)))
        elif fails_validation:
            logf = StringIO()
            logger = Logger(cast(TextIO, logf))
            self.assertFalse(is_valid(loads(json, module), logger))
            if expected:
                self.assertEqual(expected, logf.getvalue().strip('\n'))
            else:
                print(logf.getvalue())
            self.assertTrue(logger.nerrors > 0)
        else:
            with self.assertRaises(ValueError):
                is_valid(loads(json, module))



import os
import unittest

from pyjsg.validate_json import JSGPython

json = """
{
  "@context": "http://www.w3.org/ns/shex.jsonld",
  "type": "Schema",
  "shapes": [
    {
      "id": "http://a.example/S1",
      "type": "Shape",
      "expression": {
        "type": "TripleConstraint",
        "predicate": "http://a.example/p1",
        "valueExpr": {
          "type": "NodeConstraint",
          "values": [
            {
              "value": "0.0",
              "type": "http://www.w3.org/2001/XMLSchema#decimal"
            }
          ]
        }
      }
    }
  ]
}
"""


class ObjectLiteralTestCase(unittest.TestCase):
    @unittest.skipIf(False, "Must be run standalone -- context issues prevent running in test suite")
    def test_ol(self):
        """ Test that the 'type' variable in the ObjectLiteral is not mistaken as a real, unresolved type.
        """
        shexj_jsg = os.path.join(os.path.dirname(__file__), '..', '..', 'tests', 'test_basics', 'jsg', 'ShExJ.jsg')
        rval = JSGPython(shexj_jsg).conforms(json, "1val1DECIMAL")
        self.assertEqual("1val1DECIMAL: Conforms to Schema", str(rval))
        self.assertTrue(rval.success)


if __name__ == '__main__':
    unittest.main()




import unittest
from pyjsg.jsglib import JSGPattern
from tests.test_jsglib.iri_defn import IRI

class JSGPatternTest(unittest.TestCase):
    def test_wild_card(self):
        pattern = JSGPattern(r'.*')
        self.assertTrue(pattern.matches(""))
        self.assertTrue(pattern.matches("any block of text"))
        self.assertTrue(pattern.matches("\n\uFDF0*"))

    def test_fixed_values(self):
        pattern = JSGPattern(r'http:\/\/www\.w3\.org\/ns\/shex\.jsonld')
        self.assertTrue(pattern.matches("http://www.w3.org/ns/shex.jsonld"))
        self.assertFalse(pattern.matches("http://www.w3.org/ns/shex/jsonldx"))
        self.assertFalse(pattern.matches("http://www.w3.org/ns/shex/jsonld "))
        self.assertFalse(pattern.matches("http://www.w3.org/ns/shex/jsonld\n"))
        self.assertFalse(pattern.matches(" http://www.w3.org/ns/shex/jsonld"))

    def test_alternatives(self):
        pattern = JSGPattern(r'iri|bnode|nonliteral|literal')
        self.assertTrue(pattern.matches('iri'))
        self.assertTrue(pattern.matches('literal'))
        self.assertTrue(pattern.matches('nonliteral'))
        self.assertTrue(pattern.matches('bnode'))
        self.assertFalse(pattern.matches('node'))
        self.assertFalse(pattern.matches('bnod'))
        self.assertFalse(pattern.matches('IRI'))
        self.assertFalse(pattern.matches(' iri'))
        self.assertFalse(pattern.matches('iri '))

    def test_assorted_patterns(self):
        pattern = JSGPattern(r'[+-]?[0-9]+')
        self.assertTrue(pattern.matches("0"))
        self.assertTrue(pattern.matches(str(-173)))
        self.assertTrue(pattern.matches("+11720000845197308888890"))
        self.assertTrue(pattern.matches("01"))
        self.assertFalse(pattern.matches("--17"))
        self.assertFalse(pattern.matches("1.0"))

        pattern = JSGPattern(r'[+-]?[0-9]*\.[0-9]+')
        self.assertFalse(pattern.matches("0"))
        self.assertTrue(pattern.matches("0.0"))
        self.assertTrue(pattern.matches(str(float(-173))))
        self.assertTrue(pattern.matches("1.0"))
        self.assertTrue(pattern.matches("+11720000845197308888.0000000000"))

        PN_CHARS_BASE = r'[A-Z]|[a-z]|[\u00C0-\u00D6]|[\u00D8-\u00F6]|[\u00F8-\u02FF]|[\u0370-\u037D]|' \
                        r'[\u037F-\u1FFF]|[\u200C-\u200D]|[\u2070-\u218F]|[\u2C00-\u2FEF]|[\u3001-\uD7FF]|' \
                        r'[\uF900-\uFDCF]|[\uFDF0-\uFFFD]|[\u10000-\uEFFFF]'

        HEX = r'[0-9]|[A-F]|[a-f]'
        UCHAR = r'\\\\u{HEX}{HEX}{HEX}{HEX}|\\\\U{HEX}{HEX}{HEX}{HEX}{HEX}{HEX}{HEX}{HEX}'.format(HEX=HEX)
        PN_CHARS_U = r'{PN_CHARS_BASE}|_'.format(PN_CHARS_BASE=PN_CHARS_BASE)
        PN_CHARS = r'{PN_CHARS_U}|\-|[0-9]|\\u00B7|[\u0300-\u036F]|[\u203F-\u2040]'.format(PN_CHARS_U=PN_CHARS_U)
        pattern = JSGPattern(r'({PN_CHARS}|\.|\:|\/|\\\\|\#|\@|\%|\&|{UCHAR})*'.format(PN_CHARS=PN_CHARS, UCHAR=UCHAR))
        self.assertTrue(pattern.matches("http://a.example/p\u0031"))
        self.assertTrue(IRI.pattern.matches("http://a.example/p\u0031"))




if __name__ == '__main__':
    unittest.main()



import unittest

from pyjsg.validate_json import JSGPython


class BuiltinSyntaxTestCase(unittest.TestCase):
    def test_1(self):
        x = JSGPython('''doc {
    v1: @string,
    v2: @number,
    v3: @int,
    v4: @bool,
    v5: @null,
    v6: @array,
    v7: @object 
}
obj {a: . , }''')

        rslt = x.conforms('''
        { "v1": "This is text!",
          "v2": -117.432e+2,
          "v3": -100173,
          "v4": false,
          "v5": null,
          "v6": [12, "text", null],
          "v7": {"q": "life", "a": 42}
        }''')
        self.assertTrue(rslt.success)


if __name__ == '__main__':
    unittest.main()



import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()
_CONTEXT.TYPE_EXCEPTIONS.append("optFacet")
_CONTEXT.TYPE_EXCEPTIONS.append("reqFacet")
_CONTEXT.TYPE_EXCEPTIONS.append("listFacet")
_CONTEXT.TYPE_EXCEPTIONS.append("optOpt")
_CONTEXT.TYPE_EXCEPTIONS.append("optReq")
_CONTEXT.TYPE_EXCEPTIONS.append("optList")
_CONTEXT.TYPE_EXCEPTIONS.append("reqOpt")
_CONTEXT.TYPE_EXCEPTIONS.append("reqReq")
_CONTEXT.TYPE_EXCEPTIONS.append("reqList")
_CONTEXT.TYPE_EXCEPTIONS.append("listOpt")
_CONTEXT.TYPE_EXCEPTIONS.append("listReq")
_CONTEXT.TYPE_EXCEPTIONS.append("listList")


class INTEGER(jsg.JSGString):
    pattern = jsg.JSGPattern(r'[0-9]*')


class DECIMAL(jsg.JSGString):
    pattern = jsg.JSGPattern(r'[0-9]*')


class DOUBLE(jsg.JSGString):
    pattern = jsg.JSGPattern(r'[0-9]*')


class STRING(jsg.JSGString):
    pattern = jsg.JSGPattern(r'[a-z]*')


numericLiteral = typing.Union[INTEGER, DECIMAL, DOUBLE]


class optFacet(jsg.JSGObject):
    _reference_types = []
    _members = {'mininclusive': typing.Optional[numericLiteral],
                'minexclusive': typing.Optional[numericLiteral],
                'maxinclusive': typing.Optional[numericLiteral],
                'maxexclusive': typing.Optional[numericLiteral]}
    _strict = True

    def __init__(self,
                 mininclusive: typing.Optional[typing.Union[str, str, str]] = None,
                 minexclusive: typing.Optional[typing.Union[str, str, str]] = None,
                 maxinclusive: typing.Optional[typing.Union[str, str, str]] = None,
                 maxexclusive: typing.Optional[typing.Union[str, str, str]] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.mininclusive = mininclusive
        self.minexclusive = minexclusive
        self.maxinclusive = maxinclusive
        self.maxexclusive = maxexclusive



class reqFacet(jsg.JSGObject):
    _reference_types = []
    _members = {'mininclusive': numericLiteral,
                'minexclusive': numericLiteral,
                'maxinclusive': numericLiteral,
                'maxexclusive': numericLiteral}
    _strict = True

    def __init__(self,
                 mininclusive: typing.Union[str, str, str] = None,
                 minexclusive: typing.Union[str, str, str] = None,
                 maxinclusive: typing.Union[str, str, str] = None,
                 maxexclusive: typing.Union[str, str, str] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.mininclusive = mininclusive
        self.minexclusive = minexclusive
        self.maxinclusive = maxinclusive
        self.maxexclusive = maxexclusive



class listFacet(jsg.JSGObject):
    _reference_types = []
    _members = {'mininclusive': jsg.ArrayFactory('mininclusive', _CONTEXT, numericLiteral, 0, None),
                'minexclusive': jsg.ArrayFactory('minexclusive', _CONTEXT, numericLiteral, 0, None),
                'maxinclusive': jsg.ArrayFactory('maxinclusive', _CONTEXT, numericLiteral, 0, None),
                'maxexclusive': jsg.ArrayFactory('maxexclusive', _CONTEXT, numericLiteral, 0, None)}
    _strict = True

    def __init__(self,
                 mininclusive: typing.List[typing.Union[str, str, str]] = None,
                 minexclusive: typing.List[typing.Union[str, str, str]] = None,
                 maxinclusive: typing.List[typing.Union[str, str, str]] = None,
                 maxexclusive: typing.List[typing.Union[str, str, str]] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.mininclusive = mininclusive
        self.minexclusive = minexclusive
        self.maxinclusive = maxinclusive
        self.maxexclusive = maxexclusive



class optOpt(jsg.JSGObject):
    _reference_types = [optFacet]
    _members = {'first': typing.Optional[INTEGER],
                'mininclusive': typing.Optional[numericLiteral],
                'minexclusive': typing.Optional[numericLiteral],
                'maxinclusive': typing.Optional[numericLiteral],
                'maxexclusive': typing.Optional[numericLiteral],
                'last': jsg.ArrayFactory('last', _CONTEXT, STRING, 1, None)}
    _strict = True

    def __init__(self,
                 first: typing.Optional[str] = None,
                 mininclusive: typing.Optional[typing.Union[str, str, str]] = None,
                 minexclusive: typing.Optional[typing.Union[str, str, str]] = None,
                 maxinclusive: typing.Optional[typing.Union[str, str, str]] = None,
                 maxexclusive: typing.Optional[typing.Union[str, str, str]] = None,
                 last: typing.List[str] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.first = first
        self.mininclusive = mininclusive
        self.minexclusive = minexclusive
        self.maxinclusive = maxinclusive
        self.maxexclusive = maxexclusive
        self.last = last



class optReq(jsg.JSGObject):
    _reference_types = [optFacet]
    _members = {'first': jsg.Integer,
                'mininclusive': typing.Optional[numericLiteral],
                'minexclusive': typing.Optional[numericLiteral],
                'maxinclusive': typing.Optional[numericLiteral],
                'maxexclusive': typing.Optional[numericLiteral],
                'last': jsg.String}
    _strict = True

    def __init__(self,
                 first: int = None,
                 mininclusive: typing.Optional[typing.Union[str, str, str]] = None,
                 minexclusive: typing.Optional[typing.Union[str, str, str]] = None,
                 maxinclusive: typing.Optional[typing.Union[str, str, str]] = None,
                 maxexclusive: typing.Optional[typing.Union[str, str, str]] = None,
                 last: str = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.first = first
        self.mininclusive = mininclusive
        self.minexclusive = minexclusive
        self.maxinclusive = maxinclusive
        self.maxexclusive = maxexclusive
        self.last = last



class optList(jsg.JSGObject):
    _reference_types = [optFacet]
    _members = {'first': jsg.ArrayFactory('first', _CONTEXT, INTEGER, 0, None),
                'mininclusive': jsg.ArrayFactory('mininclusive', _CONTEXT, typing.Optional[numericLiteral], 0, None),
                'minexclusive': jsg.ArrayFactory('minexclusive', _CONTEXT, typing.Optional[numericLiteral], 0, None),
                'maxinclusive': jsg.ArrayFactory('maxinclusive', _CONTEXT, typing.Optional[numericLiteral], 0, None),
                'maxexclusive': jsg.ArrayFactory('maxexclusive', _CONTEXT, typing.Optional[numericLiteral], 0, None),
                'last': jsg.ArrayFactory('last', _CONTEXT, STRING, 0, None)}
    _strict = True

    def __init__(self,
                 first: typing.List[str] = None,
                 mininclusive: typing.Optional[typing.Union[str, str, str]] = None,
                 minexclusive: typing.Optional[typing.Union[str, str, str]] = None,
                 maxinclusive: typing.Optional[typing.Union[str, str, str]] = None,
                 maxexclusive: typing.Optional[typing.Union[str, str, str]] = None,
                 last: typing.List[str] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.first = first
        self.mininclusive = mininclusive
        self.minexclusive = minexclusive
        self.maxinclusive = maxinclusive
        self.maxexclusive = maxexclusive
        self.last = last



class reqOpt(jsg.JSGObject):
    _reference_types = [reqFacet]
    _members = {'first': typing.Optional[INTEGER],
                'mininclusive': typing.Optional[numericLiteral],
                'minexclusive': typing.Optional[numericLiteral],
                'maxinclusive': typing.Optional[numericLiteral],
                'maxexclusive': typing.Optional[numericLiteral],
                'last': jsg.ArrayFactory('last', _CONTEXT, STRING, 1, None)}
    _strict = True

    def __init__(self,
                 first: typing.Optional[str] = None,
                 mininclusive: typing.Union[str, str, str] = None,
                 minexclusive: typing.Union[str, str, str] = None,
                 maxinclusive: typing.Union[str, str, str] = None,
                 maxexclusive: typing.Union[str, str, str] = None,
                 last: typing.List[str] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.first = first
        self.mininclusive = mininclusive
        self.minexclusive = minexclusive
        self.maxinclusive = maxinclusive
        self.maxexclusive = maxexclusive
        self.last = last



class reqReq(jsg.JSGObject):
    _reference_types = [reqFacet]
    _members = {'first': jsg.Integer,
                'mininclusive': numericLiteral,
                'minexclusive': numericLiteral,
                'maxinclusive': numericLiteral,
                'maxexclusive': numericLiteral,
                'last': jsg.String}
    _strict = True

    def __init__(self,
                 first: int = None,
                 mininclusive: typing.Union[str, str, str] = None,
                 minexclusive: typing.Union[str, str, str] = None,
                 maxinclusive: typing.Union[str, str, str] = None,
                 maxexclusive: typing.Union[str, str, str] = None,
                 last: str = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.first = first
        self.mininclusive = mininclusive
        self.minexclusive = minexclusive
        self.maxinclusive = maxinclusive
        self.maxexclusive = maxexclusive
        self.last = last



class reqList(jsg.JSGObject):
    _reference_types = [reqFacet]
    _members = {'first': jsg.ArrayFactory('first', _CONTEXT, INTEGER, 0, None),
                'mininclusive': jsg.ArrayFactory('mininclusive', _CONTEXT, numericLiteral, 0, None),
                'minexclusive': jsg.ArrayFactory('minexclusive', _CONTEXT, numericLiteral, 0, None),
                'maxinclusive': jsg.ArrayFactory('maxinclusive', _CONTEXT, numericLiteral, 0, None),
                'maxexclusive': jsg.ArrayFactory('maxexclusive', _CONTEXT, numericLiteral, 0, None),
                'last': jsg.ArrayFactory('last', _CONTEXT, STRING, 0, None)}
    _strict = True

    def __init__(self,
                 first: typing.List[str] = None,
                 mininclusive: typing.Union[str, str, str] = None,
                 minexclusive: typing.Union[str, str, str] = None,
                 maxinclusive: typing.Union[str, str, str] = None,
                 maxexclusive: typing.Union[str, str, str] = None,
                 last: typing.List[str] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.first = first
        self.mininclusive = mininclusive
        self.minexclusive = minexclusive
        self.maxinclusive = maxinclusive
        self.maxexclusive = maxexclusive
        self.last = last



class listOpt(jsg.JSGObject):
    _reference_types = [listFacet]
    _members = {'first': typing.Optional[INTEGER],
                'mininclusive': typing.Optional[jsg.ArrayFactory('mininclusive', _CONTEXT, numericLiteral, 0, None)],
                'minexclusive': typing.Optional[jsg.ArrayFactory('minexclusive', _CONTEXT, numericLiteral, 0, None)],
                'maxinclusive': typing.Optional[jsg.ArrayFactory('maxinclusive', _CONTEXT, numericLiteral, 0, None)],
                'maxexclusive': typing.Optional[jsg.ArrayFactory('maxexclusive', _CONTEXT, numericLiteral, 0, None)],
                'last': jsg.ArrayFactory('last', _CONTEXT, STRING, 1, None)}
    _strict = True

    def __init__(self,
                 first: typing.Optional[str] = None,
                 mininclusive: typing.List[typing.Union[str, str, str]] = None,
                 minexclusive: typing.List[typing.Union[str, str, str]] = None,
                 maxinclusive: typing.List[typing.Union[str, str, str]] = None,
                 maxexclusive: typing.List[typing.Union[str, str, str]] = None,
                 last: typing.List[str] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.first = first
        self.mininclusive = mininclusive
        self.minexclusive = minexclusive
        self.maxinclusive = maxinclusive
        self.maxexclusive = maxexclusive
        self.last = last



class listReq(jsg.JSGObject):
    _reference_types = [listFacet]
    _members = {'first': jsg.Integer,
                'mininclusive': jsg.ArrayFactory('mininclusive', _CONTEXT, numericLiteral, 0, None),
                'minexclusive': jsg.ArrayFactory('minexclusive', _CONTEXT, numericLiteral, 0, None),
                'maxinclusive': jsg.ArrayFactory('maxinclusive', _CONTEXT, numericLiteral, 0, None),
                'maxexclusive': jsg.ArrayFactory('maxexclusive', _CONTEXT, numericLiteral, 0, None),
                'last': jsg.String}
    _strict = True

    def __init__(self,
                 first: int = None,
                 mininclusive: typing.List[typing.Union[str, str, str]] = None,
                 minexclusive: typing.List[typing.Union[str, str, str]] = None,
                 maxinclusive: typing.List[typing.Union[str, str, str]] = None,
                 maxexclusive: typing.List[typing.Union[str, str, str]] = None,
                 last: str = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.first = first
        self.mininclusive = mininclusive
        self.minexclusive = minexclusive
        self.maxinclusive = maxinclusive
        self.maxexclusive = maxexclusive
        self.last = last



class listList(jsg.JSGObject):
    _reference_types = [listFacet]
    _members = {'first': jsg.ArrayFactory('first', _CONTEXT, INTEGER, 0, None),
                'mininclusive': jsg.ArrayFactory('mininclusive', _CONTEXT, jsg.ArrayFactory('mininclusive', _CONTEXT, numericLiteral, 0, None), 0, None),
                'minexclusive': jsg.ArrayFactory('minexclusive', _CONTEXT, jsg.ArrayFactory('minexclusive', _CONTEXT, numericLiteral, 0, None), 0, None),
                'maxinclusive': jsg.ArrayFactory('maxinclusive', _CONTEXT, jsg.ArrayFactory('maxinclusive', _CONTEXT, numericLiteral, 0, None), 0, None),
                'maxexclusive': jsg.ArrayFactory('maxexclusive', _CONTEXT, jsg.ArrayFactory('maxexclusive', _CONTEXT, numericLiteral, 0, None), 0, None),
                'last': jsg.ArrayFactory('last', _CONTEXT, STRING, 0, None)}
    _strict = True

    def __init__(self,
                 first: typing.List[str] = None,
                 mininclusive: typing.List[typing.Union[str, str, str]] = None,
                 minexclusive: typing.List[typing.Union[str, str, str]] = None,
                 maxinclusive: typing.List[typing.Union[str, str, str]] = None,
                 maxexclusive: typing.List[typing.Union[str, str, str]] = None,
                 last: typing.List[str] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.first = first
        self.mininclusive = mininclusive
        self.minexclusive = minexclusive
        self.maxinclusive = maxinclusive
        self.maxexclusive = maxexclusive
        self.last = last


_CONTEXT.NAMESPACE = locals()



import unittest

from pyjsg.parser_impl.parser_utils import flatten, flatten_unique, as_set, is_valid_python, esc_kw


class ParserUtilsTestCase(unittest.TestCase):

    def test_flatten(self) -> None:
        self.assertEqual([], flatten([]))
        self.assertEqual([1, 2, 3, 4, 5, 6], flatten([1, [2], [3, [4, 5], [6], []]]))
        self.assertEqual(["a", "c", "a", "e", "e", "a"], flatten([["a", "c"], ["a", "e"], ["e", "a"]]))

    def test_flatten_unique(self) -> None:
        self.assertEqual([], flatten_unique([]))
        self.assertEqual([1, 2, 3, 4, 5, 6], flatten_unique([1, [2], [3, [4, 5], [[[6]]], []]]))
        self.assertEqual(["a", "c", "e"], flatten_unique([["a", "c"], ["a", "e"], ["e", "a"]]))

    def test_as_set(self) -> None:
        self.assertEqual(set(), as_set([]))
        self.assertEqual({1, 2, 3, 4, 5, 6}, as_set([1, [2], [3, [4, 5], [[[6]]], []]]))
        self.assertEqual({"a", "c", "e"}, as_set([["a", "c"], ["a", "e"], ["e", "a"]]))

    def test_is_valid_python(self) -> None:
        self.assertTrue(is_valid_python('a'))
        self.assertTrue(is_valid_python('is_valid_python'))
        self.assertFalse(is_valid_python('def'))
        self.assertFalse(is_valid_python('class'))
        self.assertFalse(is_valid_python('a 1'))
        self.assertFalse(is_valid_python('a-1'))
        self.assertFalse(is_valid_python('1a'))

    def test_esc_kw(self) -> None:
        import keyword
        self.assertEqual('a', esc_kw('a'))
        self.assertEqual('def_', esc_kw('def'))
        for k in keyword.kwlist:
            self.assertEqual(k + '_', esc_kw(k))
            self.assertEqual(k + 'x', esc_kw((k + 'x')))


if __name__ == '__main__':
    unittest.main()



import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()
_CONTEXT.TYPE_EXCEPTIONS.append("WildCard")


class WildCard(jsg.JSGObject):
    _reference_types = []
    _members = {}
    _strict = True

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)


_CONTEXT.NAMESPACE = locals()



import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()
_CONTEXT.TYPE_EXCEPTIONS.append("ShapeExternal")
_CONTEXT.TYPE_EXCEPTIONS.append("NodeConstraint")
_CONTEXT.TYPE_EXCEPTIONS.append("Shape")
_CONTEXT.TYPE_EXCEPTIONS.append("labeledShapeNot")
_CONTEXT.TYPE_EXCEPTIONS.append("ShapeOr")
_CONTEXT.TYPE_EXCEPTIONS.append("ShapeAnd")
_CONTEXT.TYPE_EXCEPTIONS.append("ShapeNot")


class _Anon1(jsg.JSGString):
    pattern = jsg.JSGPattern(r'ShapeNot')


class ShapeExternal(jsg.JSGObject):
    _reference_types = []
    _members = {}
    _strict = True

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)



class NodeConstraint(jsg.JSGObject):
    _reference_types = []
    _members = {}
    _strict = True

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)



class Shape(jsg.JSGObject):
    _reference_types = []
    _members = {}
    _strict = True

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)



shapeExprLabel = jsg.String


class labeledShapeNot(jsg.JSGObject):
    _reference_types = []
    _members = {'type': _Anon1,
                'id': shapeExprLabel,
                'shapeExpr': "shapeExpr"}
    _strict = True

    def __init__(self,
                 type: str = None,
                 id: str = None,
                 shapeExpr: typing.Union["ShapeOr", "ShapeAnd", "ShapeNot", NodeConstraint, Shape, str, ShapeExternal] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.type = type
        self.id = id
        self.shapeExpr = shapeExpr



labeledShapeExpr = typing.Union[labeledShapeNot, jsg.String]


shapeExpr = typing.Union["ShapeOr", "ShapeAnd", "ShapeNot", NodeConstraint, Shape, jsg.String, ShapeExternal]


class ShapeOr(jsg.JSGObject):
    _reference_types = []
    _members = {'shapeExprs': jsg.ArrayFactory('shapeExprs', _CONTEXT, typing.Union["ShapeOr", "ShapeAnd", "ShapeNot", NodeConstraint, Shape, jsg.String, ShapeExternal], 2, None)}
    _strict = True

    def __init__(self,
                 shapeExprs: typing.List[typing.Union["ShapeOr", "ShapeAnd", "ShapeNot", NodeConstraint, Shape, str, ShapeExternal]] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.shapeExprs = shapeExprs



class ShapeAnd(jsg.JSGObject):
    _reference_types = []
    _members = {'shapeExprs': jsg.ArrayFactory('shapeExprs', _CONTEXT, typing.Union[ShapeOr, "ShapeAnd", "ShapeNot", NodeConstraint, Shape, jsg.String, ShapeExternal], 2, None)}
    _strict = True

    def __init__(self,
                 shapeExprs: typing.List[typing.Union[ShapeOr, "ShapeAnd", "ShapeNot", NodeConstraint, Shape, str, ShapeExternal]] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.shapeExprs = shapeExprs



class ShapeNot(jsg.JSGObject):
    _reference_types = []
    _members = {'shapeExpr': shapeExpr}
    _strict = True

    def __init__(self,
                 shapeExpr: typing.Union[ShapeOr, ShapeAnd, "ShapeNot", NodeConstraint, Shape, str, ShapeExternal] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.shapeExpr = shapeExpr


_CONTEXT.NAMESPACE = locals()



import unittest
from dataclasses import dataclass
from typing import cast, Optional

from pyjsg.parser_impl.jsg_ebnf_parser import JSGEbnf
from pyjsg.parser_impl.jsg_valuetype_parser import JSGValueType

from tests.test_basics.parser import parse


@dataclass
class testentry:
    text: str
    min: int
    max: Optional[int]
    ptype: str
    oneopt: bool
    mult: bool
    stype: str

class EBNFTestCase(unittest.TestCase):
    tests = [testentry('*', 0, None, "typing.List[k]", False, True, "jsg.ArrayFactory('{name}', _CONTEXT, k, 0, None)"),
             testentry('?', 0, 1, "typing.Optional[k]", True, False, "typing.Optional[k]"),
             testentry('+', 1, None, "typing.List[k]", False, True, "jsg.ArrayFactory('{name}', _CONTEXT, k, 1, None)"),
             testentry('{0}', 0, 0, "type(None)", False, False, "type(None)"),
             testentry('{0, 1}', 0, 1, "typing.Optional[k]", True, False, "typing.Optional[k]"),
             testentry('{1}', 1, 1, "k", False, False, "k"),
             testentry('{1, }', 1, None, "typing.List[k]", False, True, "jsg.ArrayFactory('{name}', _CONTEXT, k, 1, None)"),
             testentry('{2}', 2, 2, "typing.List[k]", False, True, "jsg.ArrayFactory('{name}', _CONTEXT, k, 2, 2)"),
             testentry('{2 ,}', 2, None, "typing.List[k]", False, True, "jsg.ArrayFactory('{name}', _CONTEXT, k, 2, None)"),
             testentry('{ 3 , * }', 3, None, "typing.List[k]", False, True, "jsg.ArrayFactory('{name}', _CONTEXT, k, 3, None)"),
             testentry('{3,7}', 3, 7, "typing.List[k]", False, True, "jsg.ArrayFactory('{name}', _CONTEXT, k, 3, 7)")]

    def test_basics(self):
        ty = cast(JSGValueType, parse('"[a-z]+"', 'valueType', JSGValueType))
        for te in self.tests:
            t = cast(JSGEbnf, parse(te.text, "ebnfSuffix", JSGEbnf))
            self.assertEqual(str(t), te.text.replace(' ', ''))
            self.assertEqual(te.min, t.min, te.text)
            self.assertEqual(te.max, t.max, te.text)
            self.assertEqual(te.ptype, t.python_cardinality("k"), te.text)
            self.assertEqual(te.stype, t.signature_cardinality("k"), te.text)
            self.assertEqual(te.oneopt, t.one_optional_element, te.text)
            self.assertEqual(te.mult, t.multiple_elements, te.text)

    def test_double_optional(self):
        t = cast(JSGEbnf, parse('?', 'ebnfSuffix', JSGEbnf))
        self.assertEqual('typing.Optional[k]', t.python_cardinality('typing.Optional[k]'))
        self.assertEqual('typing.Optional[k]', t.signature_cardinality('typing.Optional[k]'))

    def test_all_optional(self):
        t = cast(JSGEbnf, parse('?', 'ebnfSuffix', JSGEbnf))
        self.assertEqual('typing.Optional[k]', t.python_cardinality('k', True))
        self.assertEqual("typing.Optional[k]", t.signature_cardinality('k', True))
        t = cast(JSGEbnf, parse('+', 'ebnfSuffix', JSGEbnf))
        self.assertEqual('typing.Optional[typing.List[k]]', t.python_cardinality('k', True))
        self.assertEqual("typing.Optional[jsg.ArrayFactory('{name}', _CONTEXT, k, 1, None)]", t.signature_cardinality('k', True))

if __name__ == '__main__':
    unittest.main()




import unittest


class Issue8TestCase(unittest.TestCase):

    def test_none_as_string_instance(self):
        from pyjsg.jsglib import JSGString, JSGPattern

        class S(JSGString):
            pattern = JSGPattern(r'[a-zA-Z]+')

        self.assertTrue(isinstance('abc', S))
        self.assertFalse(isinstance('abc1', S))
        self.assertFalse(isinstance('', S))
        self.assertFalse(isinstance(None, S))


if __name__ == '__main__':
    unittest.main()



import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()
_CONTEXT.TYPE = "type"
_CONTEXT.TYPE_EXCEPTIONS.append("ObjectLiteral")
_CONTEXT.TYPE_EXCEPTIONS.append("InnerContext")


class _Anon1(jsg.JSGString):
    pattern = jsg.JSGPattern(r'http://www\.w3\.org/ns/shex\.jsonld')


class _Anon2(jsg.JSGString):
    pattern = jsg.JSGPattern(r'http://www\.w3\.org/ns/shex\.jsonld')


class _Anon3(jsg.JSGString):
    pattern = jsg.JSGPattern(r'(iri)|(bnode)|(nonliteral)|(literal)')


class LANGTAG(jsg.JSGString):
    pattern = jsg.JSGPattern(r'[a-zA-Z]+(\-([a-zA-Z0-9])+)*')


class PN_CHARS_BASE(jsg.JSGString):
    pattern = jsg.JSGPattern(r'[A-Z]|[a-z]|[\u00C0-\u00D6]|[\u00D8-\u00F6]|[\u00F8-\u02FF]|[\u0370-\u037D]|[\u037F-\u1FFF]|[\u200C-\u200D]|[\u2070-\u218F]|[\u2C00-\u2FEF]|[\u3001-\uD7FF]|[\uF900-\uFDCF]|[\uFDF0-\uFFFD]|[\u10000-\uEFFFF]')


class HEX(jsg.JSGString):
    pattern = jsg.JSGPattern(r'[0-9]|[A-F]|[a-f]')


class PN_CHARS_U(jsg.JSGString):
    pattern = jsg.JSGPattern(r'({PN_CHARS_BASE})|_'.format(PN_CHARS_BASE=PN_CHARS_BASE.pattern))


class UCHAR(jsg.JSGString):
    pattern = jsg.JSGPattern(r'\\\\u({HEX})({HEX})({HEX})({HEX})|\\\\U({HEX})({HEX})({HEX})({HEX})({HEX})({HEX})({HEX})({HEX})'.format(HEX=HEX.pattern))


class IRIREF(jsg.JSGString):
    pattern = jsg.JSGPattern(r'([^\u0000-\u0020\u005C\u007B\u007D<>"|^`]|({UCHAR}))*'.format(UCHAR=UCHAR.pattern))


class PN_CHARS(jsg.JSGString):
    pattern = jsg.JSGPattern(r'({PN_CHARS_U})|\-|[0-9]|\\u00B7|[\u0300-\u036F]|[\u203F-\u2040]'.format(PN_CHARS_U=PN_CHARS_U.pattern))


class BNODE(jsg.JSGString):
    pattern = jsg.JSGPattern(r'_:(({PN_CHARS_U})|[0-9])((({PN_CHARS})|\.)*({PN_CHARS}))?'.format(PN_CHARS=PN_CHARS.pattern, PN_CHARS_U=PN_CHARS_U.pattern))


class PN_PREFIX(jsg.JSGString):
    pattern = jsg.JSGPattern(r'({PN_CHARS_BASE})((({PN_CHARS})|\.)*({PN_CHARS}))?'.format(PN_CHARS=PN_CHARS.pattern, PN_CHARS_BASE=PN_CHARS_BASE.pattern))


class InnerContext(jsg.JSGObject):
    _reference_types = []
    _members = {}
    _strict = False

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)



class stringFacet_1_(jsg.JSGObject):
    _reference_types = []
    _members = {'length': jsg.Integer,
                'minlength': jsg.Integer,
                'maxlength': jsg.Integer}
    _strict = True

    def __init__(self,
                 length: int = None,
                 minlength: int = None,
                 maxlength: int = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.length = length
        self.minlength = minlength
        self.maxlength = maxlength



class stringFacet_2_(jsg.JSGObject):
    _reference_types = []
    _members = {'pattern': jsg.String,
                'flags': typing.Optional[jsg.String]}
    _strict = True

    def __init__(self,
                 pattern: str = None,
                 flags: typing.Optional[str] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.pattern = pattern
        self.flags = flags



class numericFacet(jsg.JSGObject):
    _reference_types = []
    _members = {'mininclusive': jsg.Number,
                'minexclusive': jsg.Number,
                'maxinclusive': jsg.Number,
                'maxexclusive': jsg.Number,
                'totaldigits': jsg.Integer,
                'fractiondigits': jsg.Integer}
    _strict = True

    def __init__(self,
                 mininclusive: float = None,
                 minexclusive: float = None,
                 maxinclusive: float = None,
                 maxexclusive: float = None,
                 totaldigits: int = None,
                 fractiondigits: int = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.mininclusive = mininclusive
        self.minexclusive = minexclusive
        self.maxinclusive = maxinclusive
        self.maxexclusive = maxexclusive
        self.totaldigits = totaldigits
        self.fractiondigits = fractiondigits



class LiteralStem(jsg.JSGObject):
    _reference_types = []
    _members = {'stem': jsg.String}
    _strict = True

    def __init__(self,
                 stem: str = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.stem = stem



class LanguageStem(jsg.JSGObject):
    _reference_types = []
    _members = {'stem': jsg.String}
    _strict = True

    def __init__(self,
                 stem: str = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.stem = stem



class Wildcard(jsg.JSGObject):
    _reference_types = []
    _members = {}
    _strict = True

    def __init__(self,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)



class xsFacet_2_(jsg.JSGObject):
    _reference_types = [numericFacet]
    _members = {'mininclusive': jsg.Number,
                'minexclusive': jsg.Number,
                'maxinclusive': jsg.Number,
                'maxexclusive': jsg.Number,
                'totaldigits': jsg.Integer,
                'fractiondigits': jsg.Integer}
    _strict = True

    def __init__(self,
                 mininclusive: float = None,
                 minexclusive: float = None,
                 maxinclusive: float = None,
                 maxexclusive: float = None,
                 totaldigits: int = None,
                 fractiondigits: int = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.mininclusive = mininclusive
        self.minexclusive = minexclusive
        self.maxinclusive = maxinclusive
        self.maxexclusive = maxexclusive
        self.totaldigits = totaldigits
        self.fractiondigits = fractiondigits



class stringFacet(jsg.JSGObject):
    _reference_types = [stringFacet_1_, stringFacet_2_]
    _members = {'length': typing.Optional[jsg.Integer],
                'minlength': typing.Optional[jsg.Integer],
                'maxlength': typing.Optional[jsg.Integer],
                'pattern': typing.Optional[jsg.String],
                'flags': typing.Optional[jsg.String]}
    _strict = True

    def __init__(self,
                 opts_: typing.Union[stringFacet_1_, stringFacet_2_] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        if opts_ is not None:
            if isinstance(opts_, stringFacet_1_):
                self.length = opts_.length
                self.minlength = opts_.minlength
                self.maxlength = opts_.maxlength
            elif isinstance(opts_, stringFacet_2_):
                self.pattern = opts_.pattern
                self.flags = opts_.flags
            else:
                raise ValueError(f"Unrecognized value type: {opts_}")



class LiteralStemRange(jsg.JSGObject):
    _reference_types = []
    _members = {'stem': typing.Union[jsg.String, Wildcard],
                'exclusions': jsg.ArrayFactory('exclusions', _CONTEXT, typing.Union[jsg.String, LiteralStem], 1, None)}
    _strict = True

    def __init__(self,
                 stem: typing.Union[str, Wildcard] = None,
                 exclusions: typing.List[typing.Union[str, LiteralStem]] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.stem = stem
        self.exclusions = exclusions



class Language(jsg.JSGObject):
    _reference_types = []
    _members = {'languageTag': LANGTAG}
    _strict = True

    def __init__(self,
                 languageTag: str = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.languageTag = languageTag



class LanguageStemRange(jsg.JSGObject):
    _reference_types = []
    _members = {'stem': typing.Union[jsg.String, Wildcard],
                'exclusions': jsg.ArrayFactory('exclusions', _CONTEXT, typing.Union[LANGTAG, LanguageStem], 1, None)}
    _strict = True

    def __init__(self,
                 stem: typing.Union[str, Wildcard] = None,
                 exclusions: typing.List[typing.Union[str, LanguageStem]] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.stem = stem
        self.exclusions = exclusions



class xsFacet_1_(jsg.JSGObject):
    _reference_types = [stringFacet]
    _members = {'length': typing.Optional[jsg.Integer],
                'minlength': typing.Optional[jsg.Integer],
                'maxlength': typing.Optional[jsg.Integer],
                'pattern': typing.Optional[jsg.String],
                'flags': typing.Optional[jsg.String]}
    _strict = True

    def __init__(self,
                 opts_: typing.Union[stringFacet_1_, stringFacet_2_] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        if opts_ is not None:
            if isinstance(opts_, stringFacet_1_):
                self.length = opts_.length
                self.minlength = opts_.minlength
                self.maxlength = opts_.maxlength
            elif isinstance(opts_, stringFacet_2_):
                self.pattern = opts_.pattern
                self.flags = opts_.flags
            else:
                raise ValueError(f"Unrecognized value type: {opts_}")



class xsFacet(jsg.JSGObject):
    _reference_types = [xsFacet_1_, xsFacet_2_]
    _members = {'length': typing.Optional[typing.Optional[jsg.Integer]],
                'minlength': typing.Optional[typing.Optional[jsg.Integer]],
                'maxlength': typing.Optional[typing.Optional[jsg.Integer]],
                'pattern': typing.Optional[typing.Optional[jsg.String]],
                'flags': typing.Optional[typing.Optional[jsg.String]],
                'mininclusive': typing.Optional[typing.Optional[jsg.Number]],
                'minexclusive': typing.Optional[typing.Optional[jsg.Number]],
                'maxinclusive': typing.Optional[typing.Optional[jsg.Number]],
                'maxexclusive': typing.Optional[typing.Optional[jsg.Number]],
                'totaldigits': typing.Optional[typing.Optional[jsg.Integer]],
                'fractiondigits': typing.Optional[typing.Optional[jsg.Integer]]}
    _strict = True

    def __init__(self,
                 opts_: typing.Union[xsFacet_1_, xsFacet_2_] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        if opts_ is not None:
            if isinstance(opts_, xsFacet_1_):
                if opts_ is not None:
                    if isinstance(opts_, stringFacet_1_):
                        self.length = opts_.length
                        self.minlength = opts_.minlength
                        self.maxlength = opts_.maxlength
                    elif isinstance(opts_, stringFacet_2_):
                        self.pattern = opts_.pattern
                        self.flags = opts_.flags
                    else:
                        raise ValueError(f"Unrecognized value type: {opts_}")
            elif isinstance(opts_, xsFacet_2_):
                self.mininclusive = opts_.mininclusive
                self.minexclusive = opts_.minexclusive
                self.maxinclusive = opts_.maxinclusive
                self.maxexclusive = opts_.maxexclusive
                self.totaldigits = opts_.totaldigits
                self.fractiondigits = opts_.fractiondigits
            else:
                raise ValueError(f"Unrecognized value type: {opts_}")



class ObjectLiteral(jsg.JSGObject):
    _reference_types = []
    _members = {'value': jsg.String,
                'language': typing.Optional[LANGTAG],
                'type': typing.Optional[IRIREF]}
    _strict = True

    def __init__(self,
                 value: str = None,
                 language: typing.Optional[str] = None,
                 type: typing.Optional[str] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.value = value
        self.language = language
        self.type = type



class IriStem(jsg.JSGObject):
    _reference_types = []
    _members = {'stem': IRIREF}
    _strict = True

    def __init__(self,
                 stem: str = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.stem = stem



class SemAct(jsg.JSGObject):
    _reference_types = []
    _members = {'name': IRIREF,
                'code': typing.Optional[jsg.String]}
    _strict = True

    def __init__(self,
                 name: str = None,
                 code: typing.Optional[str] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.name = name
        self.code = code



shapeExprLabel = typing.Union[IRIREF, BNODE]


objectValue = typing.Union[IRIREF, ObjectLiteral]


class IriStemRange(jsg.JSGObject):
    _reference_types = []
    _members = {'stem': typing.Union[IRIREF, Wildcard],
                'exclusions': jsg.ArrayFactory('exclusions', _CONTEXT, typing.Union[IRIREF, IriStem], 1, None)}
    _strict = True

    def __init__(self,
                 stem: typing.Union[str, Wildcard] = None,
                 exclusions: typing.List[typing.Union[str, IriStem]] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.stem = stem
        self.exclusions = exclusions



tripleExprLabel = typing.Union[IRIREF, BNODE]


class ShapeExternal(jsg.JSGObject):
    _reference_types = []
    _members = {'id': typing.Optional[shapeExprLabel]}
    _strict = True

    def __init__(self,
                 id: typing.Optional[typing.Union[str, str]] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.id = id



valueSetValue = typing.Union[typing.Union[IRIREF, ObjectLiteral], IriStem, IriStemRange, LiteralStem, LiteralStemRange, Language, LanguageStem, LanguageStemRange]


class Annotation(jsg.JSGObject):
    _reference_types = []
    _members = {'predicate': IRIREF,
                'object': objectValue}
    _strict = True

    def __init__(self,
                 predicate: str = None,
                 object: typing.Union[str, ObjectLiteral] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.predicate = predicate
        self.object = object



class NodeConstraint(jsg.JSGObject):
    _reference_types = [xsFacet]
    _members = {'id': typing.Optional[shapeExprLabel],
                'nodeKind': typing.Optional[_Anon3],
                'datatype': typing.Optional[IRIREF],
                'length': typing.Optional[typing.Optional[jsg.Integer]],
                'minlength': typing.Optional[typing.Optional[jsg.Integer]],
                'maxlength': typing.Optional[typing.Optional[jsg.Integer]],
                'pattern': typing.Optional[typing.Optional[jsg.String]],
                'flags': typing.Optional[typing.Optional[jsg.String]],
                'mininclusive': typing.Optional[typing.Optional[jsg.Number]],
                'minexclusive': typing.Optional[typing.Optional[jsg.Number]],
                'maxinclusive': typing.Optional[typing.Optional[jsg.Number]],
                'maxexclusive': typing.Optional[typing.Optional[jsg.Number]],
                'totaldigits': typing.Optional[typing.Optional[jsg.Integer]],
                'fractiondigits': typing.Optional[typing.Optional[jsg.Integer]],
                'values': typing.Optional[jsg.ArrayFactory('values', _CONTEXT, typing.Union[typing.Union[IRIREF, ObjectLiteral], IriStem, IriStemRange, LiteralStem, LiteralStemRange, Language, LanguageStem, LanguageStemRange], 1, None)]}
    _strict = True

    def __init__(self,
                 id: typing.Optional[typing.Union[str, str]] = None,
                 nodeKind: typing.Optional[str] = None,
                 datatype: typing.Optional[str] = None,
                 opts_: typing.Union[xsFacet_1_, xsFacet_2_] = None,
                 values: typing.Optional[typing.List[typing.Union[typing.Union[str, ObjectLiteral], IriStem, IriStemRange, LiteralStem, LiteralStemRange, Language, LanguageStem, LanguageStemRange]]] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.id = id
        self.nodeKind = nodeKind
        self.datatype = datatype
        if opts_ is not None:
            if isinstance(opts_, xsFacet_1_):
                if opts_ is not None:
                    if isinstance(opts_, stringFacet_1_):
                        self.length = opts_.length
                        self.minlength = opts_.minlength
                        self.maxlength = opts_.maxlength
                    elif isinstance(opts_, stringFacet_2_):
                        self.pattern = opts_.pattern
                        self.flags = opts_.flags
                    else:
                        raise ValueError(f"Unrecognized value type: {opts_}")
            elif isinstance(opts_, xsFacet_2_):
                self.mininclusive = opts_.mininclusive
                self.minexclusive = opts_.minexclusive
                self.maxinclusive = opts_.maxinclusive
                self.maxexclusive = opts_.maxexclusive
                self.totaldigits = opts_.totaldigits
                self.fractiondigits = opts_.fractiondigits
            else:
                raise ValueError(f"Unrecognized value type: {opts_}")
        self.values = values



class Schema(jsg.JSGObject):
    _reference_types = []
    _members = {'@context': typing.Union[_Anon2, jsg.ArrayFactory('@context', _CONTEXT, typing.Union[_Anon1, InnerContext], 0, None)],
                'imports': typing.Optional[jsg.ArrayFactory('imports', _CONTEXT, IRIREF, 1, None)],
                'startActs': typing.Optional[jsg.ArrayFactory('startActs', _CONTEXT, SemAct, 1, None)],
                'start': typing.Optional["shapeExpr"],
                'shapes': typing.Optional[jsg.ArrayFactory('shapes', _CONTEXT, typing.Union["ShapeDecl", "ShapeOr", "ShapeAnd", "ShapeNot", NodeConstraint, "Shape", typing.Union[IRIREF, BNODE], ShapeExternal], 1, None)]}
    _strict = True

    def __init__(self,
                 imports: typing.Optional[typing.List[str]] = None,
                 startActs: typing.Optional[typing.List[SemAct]] = None,
                 start: typing.Optional[typing.Union["ShapeDecl", "ShapeOr", "ShapeAnd", "ShapeNot", NodeConstraint, "Shape", typing.Union[str, str], ShapeExternal]] = None,
                 shapes: typing.Optional[typing.List[typing.Union["ShapeDecl", "ShapeOr", "ShapeAnd", "ShapeNot", NodeConstraint, "Shape", typing.Union[str, str], ShapeExternal]]] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        setattr(self, '@context', _kwargs.get('@context', None))
        self.imports = imports
        self.startActs = startActs
        self.start = start
        self.shapes = shapes



class ShapeDecl(jsg.JSGObject):
    _reference_types = []
    _members = {'id': typing.Optional[shapeExprLabel],
                'abstract': typing.Optional[jsg.Boolean],
                'restricts': typing.Optional[jsg.ArrayFactory('restricts', _CONTEXT, typing.Union[IRIREF, BNODE], 1, None)],
                'shapeExpr': "shapeExpr"}
    _strict = True

    def __init__(self,
                 id: typing.Optional[typing.Union[str, str]] = None,
                 abstract: typing.Optional[bool] = None,
                 restricts: typing.Optional[typing.List[typing.Union[str, str]]] = None,
                 shapeExpr: typing.Union["ShapeDecl", "ShapeOr", "ShapeAnd", "ShapeNot", NodeConstraint, "Shape", typing.Union[str, str], ShapeExternal] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.id = id
        self.abstract = abstract
        self.restricts = restricts
        self.shapeExpr = shapeExpr



class ShapeOr(jsg.JSGObject):
    _reference_types = []
    _members = {'id': typing.Optional[shapeExprLabel],
                'shapeExprs': jsg.ArrayFactory('shapeExprs', _CONTEXT, typing.Union[ShapeDecl, "ShapeOr", "ShapeAnd", "ShapeNot", NodeConstraint, "Shape", typing.Union[IRIREF, BNODE], ShapeExternal], 2, None)}
    _strict = True

    def __init__(self,
                 id: typing.Optional[typing.Union[str, str]] = None,
                 shapeExprs: typing.List[typing.Union[ShapeDecl, "ShapeOr", "ShapeAnd", "ShapeNot", NodeConstraint, "Shape", typing.Union[str, str], ShapeExternal]] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.id = id
        self.shapeExprs = shapeExprs



class ShapeAnd(jsg.JSGObject):
    _reference_types = []
    _members = {'id': typing.Optional[shapeExprLabel],
                'shapeExprs': jsg.ArrayFactory('shapeExprs', _CONTEXT, typing.Union[ShapeDecl, ShapeOr, "ShapeAnd", "ShapeNot", NodeConstraint, "Shape", typing.Union[IRIREF, BNODE], ShapeExternal], 2, None)}
    _strict = True

    def __init__(self,
                 id: typing.Optional[typing.Union[str, str]] = None,
                 shapeExprs: typing.List[typing.Union[ShapeDecl, ShapeOr, "ShapeAnd", "ShapeNot", NodeConstraint, "Shape", typing.Union[str, str], ShapeExternal]] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.id = id
        self.shapeExprs = shapeExprs



class ShapeNot(jsg.JSGObject):
    _reference_types = []
    _members = {'id': typing.Optional[shapeExprLabel],
                'shapeExpr': "shapeExpr"}
    _strict = True

    def __init__(self,
                 id: typing.Optional[typing.Union[str, str]] = None,
                 shapeExpr: typing.Union[ShapeDecl, ShapeOr, ShapeAnd, "ShapeNot", NodeConstraint, "Shape", typing.Union[str, str], ShapeExternal] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.id = id
        self.shapeExpr = shapeExpr



class Shape(jsg.JSGObject):
    _reference_types = []
    _members = {'id': typing.Optional[shapeExprLabel],
                'extends': typing.Optional[jsg.ArrayFactory('extends', _CONTEXT, typing.Union[IRIREF, BNODE], 1, None)],
                'closed': typing.Optional[jsg.Boolean],
                'extra': typing.Optional[jsg.ArrayFactory('extra', _CONTEXT, IRIREF, 1, None)],
                'expression': typing.Optional["tripleExpr"],
                'semActs': typing.Optional[jsg.ArrayFactory('semActs', _CONTEXT, SemAct, 1, None)],
                'annotations': typing.Optional[jsg.ArrayFactory('annotations', _CONTEXT, Annotation, 1, None)]}
    _strict = True

    def __init__(self,
                 id: typing.Optional[typing.Union[str, str]] = None,
                 extends: typing.Optional[typing.List[typing.Union[str, str]]] = None,
                 closed: typing.Optional[bool] = None,
                 extra: typing.Optional[typing.List[str]] = None,
                 expression: typing.Optional[typing.Union["EachOf", "OneOf", "TripleConstraint", typing.Union[str, str]]] = None,
                 semActs: typing.Optional[typing.List[SemAct]] = None,
                 annotations: typing.Optional[typing.List[Annotation]] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.id = id
        self.extends = extends
        self.closed = closed
        self.extra = extra
        self.expression = expression
        self.semActs = semActs
        self.annotations = annotations



tripleExpr = typing.Union["EachOf", "OneOf", "TripleConstraint", typing.Union[IRIREF, BNODE]]


class EachOf(jsg.JSGObject):
    _reference_types = []
    _members = {'id': typing.Optional[tripleExprLabel],
                'expressions': jsg.ArrayFactory('expressions', _CONTEXT, typing.Union["EachOf", "OneOf", "TripleConstraint", typing.Union[IRIREF, BNODE]], 2, None),
                'min': typing.Optional[jsg.Integer],
                'max': typing.Optional[jsg.Integer],
                'semActs': typing.Optional[jsg.ArrayFactory('semActs', _CONTEXT, SemAct, 1, None)],
                'annotations': typing.Optional[jsg.ArrayFactory('annotations', _CONTEXT, Annotation, 1, None)]}
    _strict = True

    def __init__(self,
                 id: typing.Optional[typing.Union[str, str]] = None,
                 expressions: typing.List[typing.Union["EachOf", "OneOf", "TripleConstraint", typing.Union[str, str]]] = None,
                 min: typing.Optional[int] = None,
                 max: typing.Optional[int] = None,
                 semActs: typing.Optional[typing.List[SemAct]] = None,
                 annotations: typing.Optional[typing.List[Annotation]] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.id = id
        self.expressions = expressions
        self.min = min
        self.max = max
        self.semActs = semActs
        self.annotations = annotations



class OneOf(jsg.JSGObject):
    _reference_types = []
    _members = {'id': typing.Optional[tripleExprLabel],
                'expressions': jsg.ArrayFactory('expressions', _CONTEXT, typing.Union[EachOf, "OneOf", "TripleConstraint", typing.Union[IRIREF, BNODE]], 2, None),
                'min': typing.Optional[jsg.Integer],
                'max': typing.Optional[jsg.Integer],
                'semActs': typing.Optional[jsg.ArrayFactory('semActs', _CONTEXT, SemAct, 1, None)],
                'annotations': typing.Optional[jsg.ArrayFactory('annotations', _CONTEXT, Annotation, 1, None)]}
    _strict = True

    def __init__(self,
                 id: typing.Optional[typing.Union[str, str]] = None,
                 expressions: typing.List[typing.Union[EachOf, "OneOf", "TripleConstraint", typing.Union[str, str]]] = None,
                 min: typing.Optional[int] = None,
                 max: typing.Optional[int] = None,
                 semActs: typing.Optional[typing.List[SemAct]] = None,
                 annotations: typing.Optional[typing.List[Annotation]] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.id = id
        self.expressions = expressions
        self.min = min
        self.max = max
        self.semActs = semActs
        self.annotations = annotations



class TripleConstraint(jsg.JSGObject):
    _reference_types = []
    _members = {'id': typing.Optional[tripleExprLabel],
                'inverse': typing.Optional[jsg.Boolean],
                'predicate': IRIREF,
                'valueExpr': typing.Optional["shapeExpr"],
                'min': typing.Optional[jsg.Integer],
                'max': typing.Optional[jsg.Integer],
                'onShapeExpression': typing.Optional["shapeExpr"],
                'semActs': typing.Optional[jsg.ArrayFactory('semActs', _CONTEXT, SemAct, 1, None)],
                'annotations': typing.Optional[jsg.ArrayFactory('annotations', _CONTEXT, Annotation, 1, None)]}
    _strict = True

    def __init__(self,
                 id: typing.Optional[typing.Union[str, str]] = None,
                 inverse: typing.Optional[bool] = None,
                 predicate: str = None,
                 valueExpr: typing.Optional[typing.Union[ShapeDecl, ShapeOr, ShapeAnd, ShapeNot, NodeConstraint, Shape, typing.Union[str, str], ShapeExternal]] = None,
                 min: typing.Optional[int] = None,
                 max: typing.Optional[int] = None,
                 onShapeExpression: typing.Optional[typing.Union[ShapeDecl, ShapeOr, ShapeAnd, ShapeNot, NodeConstraint, Shape, typing.Union[str, str], ShapeExternal]] = None,
                 semActs: typing.Optional[typing.List[SemAct]] = None,
                 annotations: typing.Optional[typing.List[Annotation]] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.id = id
        self.inverse = inverse
        self.predicate = predicate
        self.valueExpr = valueExpr
        self.min = min
        self.max = max
        self.onShapeExpression = onShapeExpression
        self.semActs = semActs
        self.annotations = annotations



shapeExpr = typing.Union[ShapeDecl, ShapeOr, ShapeAnd, ShapeNot, NodeConstraint, Shape, typing.Union[IRIREF, BNODE], ShapeExternal]

_CONTEXT.NAMESPACE = locals()



import unittest

from pyjsg.validate_json import JSGPython


class MemberExampleTestCase(unittest.TestCase):
    def test1(self):
        x = JSGPython('''doc {
            last_name : @string,       # exactly one last name of type string
            first_name : @string+      # array or one or more first names
            age : @int?,               # optional age of type int
            weight : @number*          # array of zero or more weights
        }
        ''')
        rslts = x.conforms('''
        { "last_name" : "snooter",
          "first_name" : ["grunt", "peter"],
          "weight" : []
        }''')
        self.assertTrue(rslts.success)


if __name__ == '__main__':
    unittest.main()



import unittest

from tests.test_python_generator.utils import PythonGeneratorUtils


class BuiltinsTestCase(PythonGeneratorUtils):
    save_output_files = False

    def test_builtins_cooked(self):
        import tests.test_python_generator.py.builtins_cooked as doc
        jsg = '''doc {
           v1: @string,
           v2: @number,
           v3: @int,
           v4: @bool,
           v5: @null,
           v6: @array,
           v7: @object 
        }
        another_object { , }'''
        json_doc = '''
        { "v1": "This is text!",
          "v2": -117.432e+2,
          "v3": -100173,
          "v4": false,
          "v5": null,
          "v6": [12, "text", null],
          "v7": {"q": "life", "a": 42}
        }'''
        load_vals = {
            "v1": 1,
            "v2": "text",
            "v3": "text",
            "v4": "text",
            "v5": "text",
            "v6": "text",
            "v7": "text"
        }
        self.do_test(jsg, 'builtins_cooked', doc, [json_doc], load_vals)

    def test_builtins_semiraw(self):
        import tests.test_python_generator.py.builtins_cooked as doc
        jsg = '''doc {
           class: @string,
           def: @number,
           import: @int,
           with: @bool,
           if: @null,
           else: @array,
           raise: @object 
        }
        another_object { , }'''
        json_doc = '''
        { "class": "This is text!",
          "def": -117.432e+2,
          "import": -100173,
          "with": false,
          "if": null,
          "else": [12, "text", null],
          "raise": {"q": "life", "a": 42}
        }'''
        self.do_test(jsg, 'builtins_semiraw', doc, [json_doc], {})

    def test_builtins_raw(self):
        import tests.test_python_generator.py.builtins_cooked as doc
        jsg = '''doc {
           "v 1": @string,
           "v 2": @number,
           "v 3": @int,
           "v 4": @bool,
           "v 5": @null,
           "v 6": @array,
           "v 7": @object 
        }
        another_object { , }'''
        json_doc = '''
        { "v 1": "This is text!",
          "v 2": -117.432e+2,
          "v 3": -100173,
          "v 4": false,
          "v 5": null,
          "v 6": [12, "text", null],
          "v 7": {"q": "life", "a": 42}
        }'''
        self.do_test(jsg, 'builtins_raw', doc, [json_doc], {})


if __name__ == '__main__':
    unittest.main()



import unittest

from tests.utils.harness import Harness

jsg1 = '''
.TYPE t - id val ;
doc {a:.,}
id {b: @int}
val {c: @string}
'''


class MultiTypeOmissionsTestCase(Harness):
    def test_one_pass(self):
        self.do_test(jsg1, '{"t": "doc", "a": {"b": 173}}')
        self.do_test(jsg1, '{"c": "sails"}')
        self.do_test(jsg1, '{"b": -117}')

    def test_one_fail(self):
        self.do_test(jsg1, '{"t": "doc", "a": {"b": "x"}}', False, False, False, 'Invalid Integer value: "x"')
        self.do_test(jsg1, '{"c": 112}', False, False, False, 'Invalid String value "112"')
        self.do_test(jsg1, '{"b": "test"}', False, False, False, 'Invalid Integer value: "x"')
        self.do_test(jsg1, '{"d": 117}', False)


if __name__ == '__main__':
    unittest.main()



# Auto generated from JSGPython by PyJSG version 0.8b1
# Generation date: 2018-07-31 11:27
#
import typing
import pyjsg.jsglib as jsg

# .TYPE and .IGNORE settings
_CONTEXT = jsg.JSGContext()
_CONTEXT.TYPE_EXCEPTIONS.append("doc")


class doc(jsg.JSGObject):
    _reference_types = []
    _members = {'l0nl0n': jsg.ArrayFactory('l0nl0n', _CONTEXT, jsg.ArrayFactory('l0nl0n', _CONTEXT, jsg.AnyTypeFactory('l0nl0n', _CONTEXT), 0, None), 0, None)}
    _strict = True

    def __init__(self,
                 l0nl0n: typing.List[typing.List[object]] = None,
                 **_kwargs: typing.Dict[str, object]):
        super().__init__(_CONTEXT, **_kwargs)
        self.l0nl0n = l0nl0n


_CONTEXT.NAMESPACE = locals()



import unittest
from contextlib import redirect_stderr
from io import StringIO
from typing import cast

from pyjsg.parser_impl.jsg_doc_parser import JSGDocParser
from tests.test_basics.parser import parse


class ParseErrorTestCase(unittest.TestCase):
    # In the test_parser_impl unit tests, if we feed an illegal string, the parsing goes on. This test
    # is to figure out why and what to do about it
    def test_1(self):
        outf = StringIO()
        with redirect_stderr(outf):
            d = cast(JSGDocParser, parse("a a {}", "objectDef", JSGDocParser))
        self.assertIsNone(d)
        self.assertEqual("line 1:2 extraneous input 'a' expecting OBRACE", outf.getvalue().strip())


if __name__ == '__main__':
    unittest.main()
